==> fetch.php?media=snfit-2.2.2.tar.gz: snfit-2.2.2/src/model2fit.cc <==
#include <iostream>
#include <fstream>

#include "model2fit.h"
#include "snfitexception.h"
#include "magsys.h"

void Model2Fit::FixParameter(const string &paramname,double value,double error) {
  if ( ! params.LocateParam(paramname)) {
    cerr << "Parameter " << paramname << " does not exist" << endl;
    return;
  }
  LcParam* param =  const_cast<LcParam*>(params.LocateParam(paramname));
  
  param->val = value;
  param->err = error;
  /*
    IMPORTANT SO THAT MINUIT SAYS ITS FIXED
    OTHERWISE IT SAYS ITS CONSTANT

   */
  if(value!=0) {
    param->step = fabs(value/10.); // just to remove warnings of Minuit 
    param->minv = value-fabs(value/2); // just to remove warnings of Minuit 
    param->maxv = value+fabs(value/2); // just to remove warnings of Minuit 
  }else{
    param->step = 0.001;
    param->minv = -0.1;
    param->maxv = 0.1;
    
  }
  param->isfixed = true;
  // update stuff in the model if needed
  Update();
}

void Model2Fit::SetParameterValue(const string &paramname,double value) {
  LcParam* param =  const_cast<LcParam*>(params.LocateOrAddParam(paramname));
  if(!param->isfixed) {
    param->val = value;
    param->err = 0;      
  }else{
    cerr << "WARNING in Model2Fit::SetParameterValue, cannot set value of an already fixed parameter" << endl;
    return;
  }
  // update stuff in the model if needed
  Update();
}

void Model2Fit::SetParameterRange(const string &paramname, const double &min_value,const double &max_value) {
  LcParam* param =  const_cast<LcParam*>(params.LocateOrAddParam(paramname));
  
  param->minv = min_value;
  param->maxv = max_value;      
  
  cout << "Model2Fit::SetParameterRange " << min_value << " " << max_value << " for " << *param << endl;

  // update stuff in the model if needed
  Update();
}


/* used internaly to propagate param values provided by the minimizer into the
   right places */
void Model2Fit::SetParams(const double *p, const int nval) {
  
  /* NOT THE SAME NUMBER OF PARAMETERS IF SOME ARE FIXED */
  if(nval > int(params.size())) {
    cerr << "FATAL ERROR in  Model2Fit::SetParams " << endl;
    cerr << params.size() << " parameters in the model" << endl;
    cerr << nval << " parameters according to Minuit" << endl;
    abort();
  }
  for(size_t i=0; i<params.size(); ++i) params[i]->val=p[i];

  // update stuff in the model if needed
  Update();
}


bool Model2Fit::GetParamsCovMatrix(Mat &Cov) const
{
  size_t npars = params.VariableParamCount();
  Cov.allocate(npars,npars);
  // loop on params      
  size_t ki = 0;
  for (ParamCIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      const LcParam &p = **i;
      size_t kj = ki;
      for (ParamCIterator j = i; j != params.end(); ++j)
	{
	  if ((*j)->isfixed) continue;
	  const LcParam &pj = **j;
	  Cov(ki,kj) = params.GetCovariance(p.Name(), pj.Name());
	  Cov(kj,ki) = Cov(ki,kj);
	  ++kj;
	}
      ++ki;
    }
  return true;
}


bool Model2Fit::GetRestFrameMagParamGrad(const Filter& filter, double phase,
					 Vect &Grad,
					 double &Mag,
					 const string &magsys)
{
  size_t npars = params.VariableParamCount();
  Grad.allocate(npars);

  Mag = GetRestFrameMagAtPhase(filter, phase, magsys);
  if (Mag > 90) return false; // and NULL gradient
  // loop on params
  size_t ki = 0;
  for (ParamIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam &p = **i;
      double pvar = params.GetCovariance(p.Name(), p.Name());
      double eps = 0.1*sqrt(pvar);
      if(eps>0) {
      // compute gradient element
      p.val += eps;
      double off_sn_mag = GetRestFrameMagAtPhase(filter, phase, magsys);
      p.val -= eps;
      Grad(ki) = (off_sn_mag - Mag)/eps;
      }
      ki++;
    }
  return true;
}

double  Model2Fit::GetRestFrameMagAtPhase(const Filter& filter, double phase, const string &magsys) {
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFrameMagAtPhase " << endl;
#endif
  const LcParam *pRedshift = params.LocateParam("Redshift");
  const LcParam *pDayMax = params.LocateParam("DayMax");
  
  double day = phase*(1.+pRedshift->val)+pDayMax->val;
  Filter filter_red = filter;
  filter_red.RescaleX(1+pRedshift->val);
  double zp = GetMagSys(magsys)->ZeroPoint(filter);
  double flux_sn = Flux(&filter_red,day)/(1+pRedshift->val);
  if(flux_sn<=0)
    return 99;
  else
    return (-2.5*log10(flux_sn)+zp);
}
  
double  Model2Fit::GetMagAtPhase(const Filter& filter, 
				 double phase, const string &magsys) {
#ifdef DEBUG 
  cout << " Model2Fit::GetMagAtPhase " << endl;
#endif
  
  if(! CanComputeFluxFor(filter))
    return 99;
  
  const LcParam *pRedshift = params.LocateParam("Redshift");
  const LcParam *pDayMax = params.LocateParam("DayMax");

  double day = phase*(1.+pRedshift->val)+pDayMax->val;
  double zp = GetMagSys(magsys)->ZeroPoint(filter);
  double flux_sn = Flux(&filter,day);
  if(flux_sn<=0)
    return 99;
  else
    return -2.5*log10(flux_sn)+zp;
  }



bool Model2Fit::GetMagParamGrad(const Filter& filter, double phase,
					 Vect &Grad,
					 double &Mag,
					 const string &magsys)
{
  size_t npars = params.VariableParamCount();
  Grad.allocate(npars);

  Mag = GetMagAtPhase(filter, phase, magsys);
  if (Mag > 90) return false; // and NULL gradient
  // loop on params
  size_t ki = 0;
  for (ParamIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam &p = **i;
      double pvar = params.GetCovariance(p.Name(), p.Name());
      double eps = 0.1*sqrt(pvar);
      if(eps>0) {
      // compute gradient element
      p.val += eps;
      double off_sn_mag = GetMagAtPhase(filter, phase, magsys);
      p.val -= eps;
      Grad(ki) = (off_sn_mag - Mag)/eps;
      }
      ki++;
    }
  return true;
}


bool Model2Fit::GetRestFrameMagsAtPhase(const vector<Filter> &filters, double phase, 
					 FitParams &Mags,
					 const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFramesMagAtPhase " << endl;
#endif
  bool allbands = true; 
  size_t nmags = filters.size();
  Vect magvals(nmags);
 
  Mat pcov;
  GetParamsCovMatrix(pcov);
  size_t npars = pcov.SizeX();

  Mat grad(nmags,npars);

  // loop over params and bands to compute the gradient(s)
  // loop on bands
  for (size_t kb=0; kb<nmags; ++kb)
    {
      const Filter &filter = filters[kb];
      Vect grad1;
      if (!GetRestFrameMagParamGrad(filter, phase, grad1, magvals(kb), magsys))
	allbands = false;
      for (size_t kp=0; kp<grad1.Size(); kp++)
	grad(kb,kp) = grad1(kp);
    }

  Mat magsCov = grad.transposed()*pcov*grad;

  // convert magvals & Cov to the FitParams structure (for IO's)
  vector<string> names;
  for (size_t k1=0; k1<nmags; ++k1)
    {
      char name[100];
      sprintf(name,"RestFrameMag_%d_%s",(int)phase,filters[k1].Band().c_str());
      LcParam *p = Mags.LocateOrAddNonConstParam(name);
      p->val = magvals(k1);
      if (p->val < 90)
	p->err = sqrt(magsCov(k1,k1));
      else
	p->err = 99;
      names.push_back(name);
      for (size_t k2=0; k2<k1; ++k2)
	Mags.SetCovariance(name, names[k2], magsCov(k1,k2));
    }

  return allbands;
}

bool Model2Fit::GetRestFrameMagAndCov(const Filter &filter, double phase, 
				      FitParams &MagAndP,
				      const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFrameMagAndCov " << endl;
#endif
  MagAndP = params; // return the current stuff if we cannot compute the magnitude
  double mag;
  Vect grad;  
  if (!GetRestFrameMagParamGrad(filter, phase, grad, mag, magsys)) return false;
  

  // get the covariance matrix
  Mat pcov;
  GetParamsCovMatrix(pcov);
  
  Vect cov_m_p = pcov*grad;

  char magName[100];
  sprintf(magName,"RestFrameMag_%d_%s",(int)phase,filter.Band().c_str());
  LcParam *pmag = MagAndP.LocateOrAddNonConstParam(magName);
  pmag->val = mag;

  double magvar = 0;
  size_t ki = 0;
  for (ParamIterator i=MagAndP.begin(); i != MagAndP.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam *p = *i;
      if (p==pmag) continue;
      MagAndP.SetCovariance(p->Name(), magName, cov_m_p(ki));
      magvar += cov_m_p(ki)*grad(ki);
      ++ki;
    }
  pmag->err = sqrt(magvar);
  return true;
}

bool Model2Fit::GetMagsAtPhase(const vector<Filter> &filters, double phase, 
					 FitParams &Mags,
					 const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetMagsAtPhase " << endl;
#endif
  bool allbands = true; 
  size_t nmags = filters.size();
  Vect magvals(nmags);
  
  Mat pcov;
  GetParamsCovMatrix(pcov);
  size_t npars = pcov.SizeX();

  Mat grad(nmags,npars);

  // loop over params and bands to compute the gradient(s)
  // loop on bands
  for (size_t kb=0; kb<nmags; ++kb)
    {
      const Filter &filter = filters[kb];
      Vect grad1;
      if (!GetMagParamGrad(filter, phase, grad1, magvals(kb), magsys))
	allbands = false;
      for (size_t kp=0; kp<grad1.Size(); kp++)
	grad(kb,kp) = grad1(kp);
    }

  Mat magsCov = grad.transposed()*pcov*grad;

  // convert magvals & Cov to the FitParams structure (for IO's)
  vector<string> names;
  for (size_t k1=0; k1<nmags; ++k1)
    {
      char name[100];
      sprintf(name,"Mag_%d_%s",(int)phase,filters[k1].Band().c_str());
      LcParam *p = Mags.LocateOrAddNonConstParam(name);
      p->val = magvals(k1);
      if (p->val < 90)
	p->err = sqrt(magsCov(k1,k1));
      else
	p->err = 99;
      names.push_back(name);
      for (size_t k2=0; k2<k1; ++k2)
	Mags.SetCovariance(name, names[k2], magsCov(k1,k2));
    }

  return allbands;
}

void Model2Fit::ReadParams(const string& filename)
{
  params.read(filename);
}


#include "saltmodel.h"
#include "salt2model.h"

// the "virtual constructor" for derived classes

Model2Fit *NewModel2Fit(const string &FileName)
{
  vector<string> fitParamNames = FitParamNames(FileName);
  Model2Fit *p = NULL;
  for (size_t k=0; k< fitParamNames.size(); ++k)
    {
      if (fitParamNames[k] == "Salt2Model") p = new Salt2Model();
      else if (fitParamNames[k] == "SaltModel") p = new SaltModel();
    }
  if (p) p->ReadParams(FileName);
  else throw(SnfitException("NewModel2Fit : could not find a LC model " ));
  return p;
}

// routines that are not implemented for base class
// in .cc to prevent warnings at compilation

double Model2Fit::SpectrumFlux(const double observerlambda, const double observerday) const 
{return 0;}
double Model2Fit::SpectrumModelError(const double observerlambda, const double observerday, bool with_scaling) 
{return 0;}  
double Model2Fit::GalaxySpectrumFlux(const double observerlambda, bool only_galaxy)
{return 0;}
#ifndef NEW_ERRORS
double Model2Fit::ModelKError(const Filter* filter) 
{return 0;}
#else
double Model2Fit::ModelRelativeCovariance(const Filter* filter,const double observerday1,const double observerday2) 
{return 0;};
#endif
double Model2Fit::ModelRelativeError(const Filter* filter,const double observerday, bool with_scaling) 
{return 0;};
vector<LcParam*> Model2Fit::InitCalibrationParams(const MeasuredSpectrum *spec, int n_recalibration_params)
{vector<LcParam*> V; return V;}
double Model2Fit::SpectrumCalibration(const double observerlambda, 
				      const MeasuredSpectrum& spectrum) 
{return 1;};

==> fetch.php?media=snfit-2.4.1.tar.gz: snfit-2.4.2/src/model2fit.cc <==
#include <iostream>
#include <fstream>

#include "model2fit.h"
#include "snfitexception.h"
#include "magsys.h"

bool Model2Fit::FixParameter(const string &paramname,double value,double error) {

  LcParam* param =  const_cast<LcParam*>(params.LocateParam(paramname));
  if (!param) {
    cerr << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << " list of parameters :  " << endl << params << endl;
    return false;
  }
  
  param->val = value;
  param->err = error;
  /*
    IMPORTANT SO THAT MINUIT SAYS ITS FIXED
    OTHERWISE IT SAYS ITS CONSTANT

   */
  if(value!=0) {
    param->step = fabs(value/10.); // just to remove warnings of Minuit 
    param->minv = value-fabs(value/2); // just to remove warnings of Minuit 
    param->maxv = value+fabs(value/2); // just to remove warnings of Minuit 
  }else{
    param->step = 0.001;
    param->minv = -0.1;
    param->maxv = 0.1;
    
  }
  param->isfixed = true;
  // update stuff in the model if needed
  Update();
  return true;
}

bool Model2Fit::SetParameterValue(const string &paramname,double value) {
  LcParam* param =  const_cast<LcParam*>(params.LocateOrAddParam(paramname));
  if (!param)
    {
    cerr << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << " list of parameters :  " << endl << params << endl;
    return false;
    }      
  if(!param->isfixed) {
    param->val = value;
    param->err = 0;      
  }else{
    cerr << "WARNING in Model2Fit::SetParameterValue, cannot set value of an already fixed parameter" << endl;
    return false;
  }
  // update stuff in the model if needed
  Update();
  return true;
}

bool Model2Fit::SetParameterRange(const string &paramname, const double &min_value,const double &max_value) {
  LcParam* param =  const_cast<LcParam*>(params.LocateOrAddParam(paramname));
  if (!param)
    {
    cerr << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << "Parameter " << paramname << " does not exist in model " << ModelName() << endl;
    cout << " list of parameters :  " << endl << params << endl;
    return false;
    }      
  
  param->minv = min_value;
  param->maxv = max_value;      
  
  cout << "Model2Fit::SetParameterRange " << min_value << " " << max_value << " for " << *param << endl;

  // update stuff in the model if needed
  Update();
  return true;
}


/* used internaly to propagate param values provided by the minimizer into the
   right places */
void Model2Fit::SetParams(const double *p, const int nval) {
  
  /* NOT THE SAME NUMBER OF PARAMETERS IF SOME ARE FIXED */
  if(nval > int(params.size())) {
    cerr << "FATAL ERROR in  Model2Fit::SetParams " << endl;
    cerr << params.size() << " parameters in the model" << endl;
    cerr << nval << " parameters according to Minuit" << endl;
    abort();
  }
  for(size_t i=0; i<params.size(); ++i) params[i]->val=p[i];

  // update stuff in the model if needed
  Update();
}


bool Model2Fit::GetParamsCovMatrix(Mat &Cov) const
{
  size_t npars = params.VariableParamCount();
  Cov.allocate(npars,npars);
  // loop on params      
  size_t ki = 0;
  for (ParamCIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      const LcParam &p = **i;
      size_t kj = ki;
      for (ParamCIterator j = i; j != params.end(); ++j)
	{
	  if ((*j)->isfixed) continue;
	  const LcParam &pj = **j;
	  Cov(ki,kj) = params.GetCovariance(p.Name(), pj.Name());
	  Cov(kj,ki) = Cov(ki,kj);
	  ++kj;
	}
      ++ki;
    }
  return true;
}


bool Model2Fit::GetRestFrameMagParamGrad(const Filter& filter, double phase,
					 Vect &Grad,
					 double &Mag,
					 const string &magsys)
{
  size_t npars = params.VariableParamCount();
  Grad.allocate(npars);

  Mag = GetRestFrameMagAtPhase(filter, phase, magsys);
  if (Mag > 90) return false; // and NULL gradient
  // loop on params
  size_t ki = 0;
  for (ParamIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam &p = **i;
      double pvar = params.GetCovariance(p.Name(), p.Name());
      double eps = 0.1*sqrt(pvar);
      if(eps>0) {
      // compute gradient element
      p.val += eps;
      double off_sn_mag = GetRestFrameMagAtPhase(filter, phase, magsys);
      p.val -= eps;
      Grad(ki) = (off_sn_mag - Mag)/eps;
      }
      ki++;
    }
  return true;
}

double  Model2Fit::GetRestFrameMagAtPhase(const Filter& filter, double phase, const string &magsys) {
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFrameMagAtPhase " << endl;
#endif
  const LcParam *pRedshift = params.LocateParam("Redshift");
  const LcParam *pDayMax = params.LocateParam("DayMax");
  
  double day = phase*(1.+pRedshift->val)+pDayMax->val;
  Filter filter_red = filter;
  filter_red.RescaleX(1+pRedshift->val);
  double zp = GetMagSys(magsys)->ZeroPoint(filter);
  double flux_sn = Flux(&filter_red,day)/(1+pRedshift->val);
  if(flux_sn<=0)
    return 99;
  else
    return (-2.5*log10(flux_sn)+zp);
}
  
double  Model2Fit::GetMagAtPhase(const Filter& filter, 
				 double phase, const string &magsys) {
#ifdef DEBUG 
  cout << " Model2Fit::GetMagAtPhase " << endl;
#endif
  
  if(! CanComputeFluxFor(filter))
    return 99;
  
  const LcParam *pRedshift = params.LocateParam("Redshift");
  const LcParam *pDayMax = params.LocateParam("DayMax");

  double day = phase*(1.+pRedshift->val)+pDayMax->val;
  double zp = GetMagSys(magsys)->ZeroPoint(filter);
  double flux_sn = Flux(&filter,day);
  if(flux_sn<=0)
    return 99;
  else
    return -2.5*log10(flux_sn)+zp;
  }



bool Model2Fit::GetMagParamGrad(const Filter& filter, double phase,
					 Vect &Grad,
					 double &Mag,
					 const string &magsys)
{
  size_t npars = params.VariableParamCount();
  Grad.allocate(npars);

  Mag = GetMagAtPhase(filter, phase, magsys);
  if (Mag > 90) return false; // and NULL gradient
  // loop on params
  size_t ki = 0;
  for (ParamIterator i=params.begin(); i != params.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam &p = **i;
      double pvar = params.GetCovariance(p.Name(), p.Name());
      double eps = 0.1*sqrt(pvar);
      if(eps>0) {
      // compute gradient element
      p.val += eps;
      double off_sn_mag = GetMagAtPhase(filter, phase, magsys);
      p.val -= eps;
      Grad(ki) = (off_sn_mag - Mag)/eps;
      }
      ki++;
    }
  return true;
}


bool Model2Fit::GetRestFrameMagsAtPhase(const vector<Filter> &filters, double phase, 
					 FitParams &Mags,
					 const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFramesMagAtPhase " << endl;
#endif
  bool allbands = true; 
  size_t nmags = filters.size();
  Vect magvals(nmags);
 
  Mat pcov;
  GetParamsCovMatrix(pcov);
  size_t npars = pcov.SizeX();

  Mat grad(nmags,npars);

  // loop over params and bands to compute the gradient(s)
  // loop on bands
  for (size_t kb=0; kb<nmags; ++kb)
    {
      const Filter &filter = filters[kb];
      Vect grad1;
      if (!GetRestFrameMagParamGrad(filter, phase, grad1, magvals(kb), magsys))
	allbands = false;
      for (size_t kp=0; kp<grad1.Size(); kp++)
	grad(kb,kp) = grad1(kp);
    }

  Mat magsCov = grad.transposed()*pcov*grad;

  // convert magvals & Cov to the FitParams structure (for IO's)
  vector<string> names;
  for (size_t k1=0; k1<nmags; ++k1)
    {
      char name[100];
      sprintf(name,"RestFrameMag_%d_%s",(int)phase,filters[k1].Band().c_str());
      LcParam *p = Mags.LocateOrAddNonConstParam(name);
      p->val = magvals(k1);
      if (p->val < 90)
	p->err = sqrt(magsCov(k1,k1));
      else
	p->err = 99;
      names.push_back(name);
      for (size_t k2=0; k2<k1; ++k2)
	Mags.SetCovariance(name, names[k2], magsCov(k1,k2));
    }

  return allbands;
}

bool Model2Fit::GetRestFrameMagAndCov(const Filter &filter, double phase, 
				      FitParams &MagAndP,
				      const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetRestFrameMagAndCov " << endl;
#endif
  MagAndP = params; // return the current stuff if we cannot compute the magnitude
  double mag;
  Vect grad;  
  if (!GetRestFrameMagParamGrad(filter, phase, grad, mag, magsys)) return false;
  

  // get the covariance matrix
  Mat pcov;
  GetParamsCovMatrix(pcov);
  
  Vect cov_m_p = pcov*grad;

  char magName[100];
  sprintf(magName,"RestFrameMag_%d_%s",(int)phase,filter.Band().c_str());
  LcParam *pmag = MagAndP.LocateOrAddNonConstParam(magName);
  pmag->val = mag;

  double magvar = 0;
  size_t ki = 0;
  for (ParamIterator i=MagAndP.begin(); i != MagAndP.end(); ++i)
    {
      if ((*i)->isfixed) continue;
      LcParam *p = *i;
      if (p==pmag) continue;
      MagAndP.SetCovariance(p->Name(), magName, cov_m_p(ki));
      magvar += cov_m_p(ki)*grad(ki);
      ++ki;
    }
  pmag->err = sqrt(magvar);
  return true;
}

bool Model2Fit::GetMagsAtPhase(const vector<Filter> &filters, double phase, 
					 FitParams &Mags,
					 const string &magsys)
{
#ifdef DEBUG 
  cout << " Model2Fit::GetMagsAtPhase " << endl;
#endif
  bool allbands = true; 
  size_t nmags = filters.size();
  Vect magvals(nmags);
  
  Mat pcov;
  GetParamsCovMatrix(pcov);
  size_t npars = pcov.SizeX();

  Mat grad(nmags,npars);

  // loop over params and bands to compute the gradient(s)
  // loop on bands
  for (size_t kb=0; kb<nmags; ++kb)
    {
      const Filter &filter = filters[kb];
      Vect grad1;
      if (!GetMagParamGrad(filter, phase, grad1, magvals(kb), magsys))
	allbands = false;
      for (size_t kp=0; kp<grad1.Size(); kp++)
	grad(kb,kp) = grad1(kp);
    }

  Mat magsCov = grad.transposed()*pcov*grad;

  // convert magvals & Cov to the FitParams structure (for IO's)
  vector<string> names;
  for (size_t k1=0; k1<nmags; ++k1)
    {
      char name[100];
      sprintf(name,"Mag_%d_%s",(int)phase,filters[k1].Band().c_str());
      LcParam *p = Mags.LocateOrAddNonConstParam(name);
      p->val = magvals(k1);
      if (p->val < 90)
	p->err = sqrt(magsCov(k1,k1));
      else
	p->err = 99;
      names.push_back(name);
      for (size_t k2=0; k2<k1; ++k2)
	Mags.SetCovariance(name, names[k2], magsCov(k1,k2));
    }

  return allbands;
}

void Model2Fit::ReadParams(const string& filename)
{
  params.read(filename);
}

/* SZF */
GlobalVal Model2Fit::CalcDM15(const Filter& observerfilter){
  /* Calculates the maximum flux on observerfilter, and the corresponding date of max. 
   * Calculates dm15 for this band too.
   * Meant to be used in codes like snlc.
   * 
   * Output: GlobalVal with keys:
   *        BAND_DayMax : mjd of the maximum in this band
   *        BAND_phase_DayMax : phase of the maximum in this band
   *        BAND_FluxMax : maximum flux in this band
   *        BAND_FluxMax_err : error on the maximum flux in this band
   *        BAND_dm15 : dm15 in this band (i.e. the maginitude difference between BAND_FluxMax
   *                    and the flux at BAND_phase_DayMax + 15
   *        
   *        BAND_dm15_err : error on dm15
   *
   * FIXME:
   *       - flux_scale_sim2data is not taken into account. Is that ok ?
   */
  
    /* Calculate date of max for this band and the dm15 
    * This is dumb brute force, can be improved. 
    */

  double& redshift = params.LocateOrAddNonConstParam("Redshift")->val;
  const double phase_factor_z_new = (1. + redshift);

  double DayMax = params.LocateParam("DayMax")->val;
  double delta_phase = 10.; // look for max in a window +- 10 days around DayMax
  int n_step = 10;
  double phase_step = 2.*delta_phase/n_step;
  double phase_step_min = 0.001; // Target accuracy of the maximum measurement
  
  double phase_DayMax=0.;
  double flux_max;
  
  while(phase_step > phase_step_min){
    for (double phase = phase_DayMax - delta_phase ; phase < phase_DayMax+delta_phase+phase_step/2.; phase += phase_step) {
      double day = phase * phase_factor_z_new + DayMax;
      double flux = Flux(&observerfilter,day);
      if (flux > flux_max){
        flux_max = flux;
        phase_DayMax = phase;
      }
    }
    cout << "phase_DayMax: " << phase_DayMax << endl;
    delta_phase = phase_step;
    phase_step = 2.*delta_phase/n_step;
  }
  double day_max = phase_DayMax * phase_factor_z_new + DayMax;
  double day_15 = (phase_DayMax + 15.) * phase_factor_z_new + DayMax;
  
  double flux_max_error =  fabs(ModelRelativeError(&observerfilter,day_max,true) * flux_max);    
  double flux_m15 = Flux(&observerfilter,day_15);
  double flux_m15_error =  fabs(ModelRelativeError(&observerfilter, day_15,true) * flux_m15);    
    
  /* FIXME: error calculation doesn't take into account the error on the phase 
   * Note that for data in mag, zp = 0. (see measureddata.cc line 236
   */
  double dm15 = -2.5 * log(flux_max/flux_m15);
  double dm15_error = ModelRelativeError(&observerfilter,day_max,true);
  dm15_error +=  ModelRelativeError(&observerfilter,day_max + 15,true) ;
  dm15_error *= 2.5;
  
  GlobalVal globalval_output;
  globalval_output.AddKey("BAND_DayMax", day_max);
  globalval_output.AddKey("BAND_phase_DayMax", phase_DayMax);
  globalval_output.AddKey("BAND_FluxMax", flux_max);
  globalval_output.AddKey("BAND_FluxMax_err", flux_max_error );
  globalval_output.AddKey("BAND_dm15", dm15);
  globalval_output.AddKey("BAND_dm15_err", dm15_error );
  
  return globalval_output;
}



#include "saltmodel.h"
#include "salt2model.h"
#include "saltnirmodel.h"

// the "virtual constructor" for derived classes

Model2Fit *SnModel2Fit(const string &ModelName, const bool Warn)
{
  if (ModelName == "Salt2Model") return new Salt2Model();
  if (ModelName  == "SnoopyModel") return new SaltNirModel();
  if (ModelName  == "SaltNirModel") return new SaltNirModel();
  if (ModelName  == "SaltModel") return new SaltModel();
  if (Warn)
    {
      cout << "WARNING :  SnModel2Fit : I don't have a model named "+ModelName << endl;
      cout << "WARNING:  Available models :  Salt2Model , SaltNirModel , SaltModel" << endl;
    }
  return NULL;
}

  


Model2Fit *NewModel2Fit(const string &FileName)
{

  vector<string> fitParamNames = FitParamNames(FileName);
  Model2Fit *p = NULL;
  for (size_t k=0; k< fitParamNames.size(); ++k)
    {
      p = SnModel2Fit(fitParamNames[k], false /* No Warning */);
      if (p) break;
    }
  if (p) p->ReadParams(FileName);
  else throw(SnfitException("NewModel2Fit : could not find a LC model in "+FileName ));
  return p;
}



// routines that are not implemented for base class
// in .cc to prevent warnings at compilation

double Model2Fit::SpectrumFlux(const double observerlambda, const double observerday) const 
{return 0;}
double Model2Fit::SpectrumModelError(const double observerlambda, const double observerday, bool with_scaling) 
{return 0;}  
double Model2Fit::GalaxySpectrumFlux(const double observerlambda, bool only_galaxy)
{return 0;}
#ifndef NEW_ERRORS
double Model2Fit::ModelKError(const Filter* filter) 
{return 0;}
#else
double Model2Fit::ModelRelativeCovariance(const Filter* filter,const double observerday1,const double observerday2) 
{return 0;};
#endif
double Model2Fit::ModelRelativeError(const Filter* filter,const double observerday, bool with_scaling) 
{return 0;};
vector<LcParam*> Model2Fit::InitCalibrationParams(const MeasuredSpectrum *spec, int n_recalibration_params)
{vector<LcParam*> V; return V;}
double Model2Fit::SpectrumCalibration(const double observerlambda, 
				      const MeasuredSpectrum& spectrum) 
{return 1;};
